ROOT_DEVICES = "15001"
ROOT_GROUPS = "15004"
ROOT_MOODS = "15005"
ROOT_SMART_TASKS = "15010"
ROOT_START_ACTION = "15013"  # found under ATTR_START_ACTION
PATH_GATEWAY_INFO = ["15011", "15012"]

ATTR_APPLICATION_TYPE = "5750"
ATTR_DEVICE_INFO = "3"
ATTR_NAME = "9001"
ATTR_CREATED_AT = "9002"
ATTR_ID = "9003"
ATTR_REACHABLE_STATE = "9019"
ATTR_LAST_SEEN = "9020"
ATTR_LIGHT_CONTROL = "3311"  # array

ATTR_NTP = "9023"
ATTR_FIRMWARE_VERSION = "9029"
ATTR_CURRENT_TIME_UNIX = "9059"
ATTR_CURRENT_TIME_ISO8601 = "9060"
ATTR_FIRST_SETUP = "9069"  # ??? unix epoch value when gateway first setup
ATTR_GATEWAY_ID = "9081"  # ??? id of the gateway

ATTR_LIGHT_STATE = "5850"  # 0 / 1
ATTR_LIGHT_DIMMER = "5851"  # Dimmer, not following spec: 0..255
ATTR_LIGHT_COLOR = "5706"  # string representing a value in some color space
ATTR_LIGHT_COLOR_X = "5709"
ATTR_LIGHT_COLOR_Y = "5710"

ATTR_START_ACTION = "9042"  # array

ATTR_SMART_TASK_TYPE = "9040"  # 4 = transition | 1 = not home | 2 = on/off
ATTR_SMART_TASK_NOT_AT_HOME = 1
ATTR_SMART_TASK_LIGHTS_OFF = 2
ATTR_SMART_TASK_WAKE_UP = 4

ATTR_SMART_TASK_TRIGGER_TIME_INTERVAL = "9044"
ATTR_SMART_TASK_TRIGGER_TIME_START_HOUR = "9046"
ATTR_SMART_TASK_TRIGGER_TIME_START_MIN = "9047"

ATTR_TRANSITION_TIME = "5712"
ATTR_REPEAT_DAYS = "9041"
